@@ -14,12 +14,20 @@ package stdouthandle;
 #####################################################################
 
 require Exporter;
-use Fcntl qw(:seek);
+my $include_imp_module = 'use Fcntl;';
+eval $include_imp_module;
+if($@ ne '')
+ {
+  # Better than nothing :)
+  use constant SEEK_SET => 0;
+  use constant SEEK_CUR => 1;
+  use constant SEEK_END => 2;
+ }
 use vars qw($VERSION @ISA @EXPORT);
 @ISA = qw(Exporter);
 @EXPORT = qw(reset clear $sys_stdouthandle_print_text $sys_stdouthandle_content_ok 
              $sys_stdouthandle_header $sys_stdouthandle_header_up_to_now);
-$VERSION = "1.26";
+$VERSION = "1.27";
 
 $sys_stdouthandle_print_text = 0;
 $sys_stdouthandle_header = 0;
@@ -58,7 +66,7 @@ sub WRITE
        {
        	CORE::print "Content-type: text/html\n";
        }
-      CORE::print "X-Powered-By: WebTools/1.26\n\n";
+      CORE::print "X-Powered-By: WebTools/1.27\n\n";
       $sys_stdouthandle_content_ok = 1;
       $sys_stdouthandle_header = 1;
      }
@@ -90,7 +98,7 @@ sub PRINT
        {
        	CORE::print "Content-type: text/html\n";
        }
-      CORE::print "X-Powered-By: WebTools/1.26\n\n";
+      CORE::print "X-Powered-By: WebTools/1.27\n\n";
       $sys_stdouthandle_content_ok = 1;
       $sys_stdouthandle_header = 1;
      }
@@ -122,7 +130,7 @@ sub PRINTF
        {
        	CORE::print "Content-type: text/html\n";
        }
-      CORE::print "X-Powered-By: WebTools/1.26\n\n";
+      CORE::print "X-Powered-By: WebTools/1.27\n\n";
       $sys_stdouthandle_content_ok = 1;
       $sys_stdouthandle_header = 1;
      }
